
-- G.J. Parker (c) 2019
-- simple AppleScript routines for dealing w/ icons on desktop

script ApplescriptBridge
    
    property parent : class "NSObject"
    property iconSet : {}         -- this data is shared w/ swift app- swift doesn't know what it is and we don't care
    property numDesktop : {}      -- just the number of items on the the Desktop, swift wants to know to update some user info
    property numArrangement : {}  -- just the number of items stored in iconSet, swift wants to know to update some user info
    
    
    on memorize() -- memorize icon name/positions (and screen size, icon size and text size)
        tell application "Finder" -- go get names, positions, current screen size, icon size and text size...
            set iconNames to name of items of desktop
            set iconPositions to desktop position of items of desktop
            copy the bounds of the desktop's window to the screenSize
            set iconSize to icon size of (icon view options of desktop's window)
            set textSize to text size of (icon view options of desktop's window)
        end tell
        
        set iconSet to {iconNames, iconPositions, screenSize, iconSize, textSize} -- array of arrays
        set numDesktop to (count of iconNames)  -- we have this, might as well store it
        copy numDesktop to numArrangement
    end memorize
    
    on restore() -- given iconSet, restore Desktop icons to the correct postions (along w/ icon size and text size). if screen size changed, scale the postions
        if count of iconSet > 0 then
            set iconNames to item 1 of iconSet
            set iconPositions to item 2 of iconSet
            set screenSize to item 3 of iconSet
            set iconSize to item 4 of iconSet
            set textSize to item 5 of iconSet
            set numArrangement to (count of iconNames) -- might as well store it
            
            tell application "Finder" -- find current screen size and get rid of arranged by if screen size changed
                copy the bounds of the desktop's window to the newScreenSize -- get current resolution
                if arrangement of (icon view options of desktop's window)  not arranged and newScreenSize  screenSize then
                    set arrangement of (icon view options of desktop's window) to not arranged -- we have to turn off Snap to Grid or whatever else if resolution changed
                end if
                set newIconSize to icon size of (icon view options of desktop's window)
                set newTextSize to text size of (icon view options of desktop's window)
            end tell
            
            if iconSize  newIconSize  then -- if icon sizes change, change it back (may require a Finder restart)
                ignoring application responses
                    tell application "Finder" to set icon size of (icon view options of desktop's window) to iconSize
                end ignoring
            end if
            
            if textSize  newTextSize then -- if text sizes change, change it back (may require a Finder restart)
                ignoring application responses
                    tell application "Finder" to set text size of (icon view options of desktop's window) to textSize
                end ignoring
            end if
            
            set newdx to item 1 of newScreenSize as integer -- scaling parameters incase screen size changed
            set newdy to item 2 of newScreenSize as integer
            set xrel to ((item 3 of newScreenSize as integer) - newdx) / ((item 3 of screenSize as integer) - (item 1 of screenSize as integer))
            set yrel to ((item 4 of newScreenSize as integer) - newdy) / ((item 4 of screenSize as integer) - (item 2 of screenSize as integer))
            
            repeat with i from 1 to number of items of iconNames -- for each desktop icon, compute and put back at the old position
                ignoring application responses
                    tell application "Finder" to set desktop position of item (item i of iconNames as string) of desktop to {newdx + (0.5 + ((item 1 of (item i of iconPositions)) as integer - newdx) * xrel) div 1, newdy + (0.5 + ((item 2 of (item i of iconPositions)) as integer - newdy) * yrel) div 1} -- new position
                end ignoring
            end repeat
            
        end if
    end restore
    
    on showNewIcons()
        if count of iconSet > 0 then
            set iconNames to item 1 of iconSet
            
            set oldNames to {} -- force list to be a list of strings - bridge may confuse things...
            repeat with i from 1 to number of items of iconNames
                set end of oldNames to item i of iconNames as string
            end repeat
            
            tell application "Finder" to set newIconNames to name of items of desktop  -- get current list of icon names
            set numDesktop to (count of newIconNames) -- might as well update numDesktop...
            
            set newIcons to {}  -- find new icons
            repeat with x in newIconNames
                if {x as string} is not in oldNames then set end of newIcons to x
            end repeat
            
            if count of newIcons > 0 then -- if there are new icons, select them and get out
                set sel to {}
                tell application "Finder"
                    repeat with aName in newIcons
                        set end of sel to item aName of desktop
                    end repeat
                    activate
                    activate window of desktop
                    set selection to sel
                end tell
                else -- no new icons, clear any selection and then beep
                tell application "Finder"
                    activate
                    activate window of desktop
                    set selection to {}
                end tell
                beep
            end if
            tell application "DIM 3.0" to activate -- and regain focus
        end if
    end showNewIcons
    
    on numOnDesktop() -- set the current count of icons on Desktop
        tell application "Finder" to set numDesktop to (count desktop's items)
    end numOnDesktop
    
    on numInSet() -- set current count of items in iconSet
        set numArrangement to count (item 1 of iconSet)
    end numInSet
    
    on testBridge()
        set test to "no"
        tell application "Finder" to set test to arrangement of (icon view options of desktop's window) as string
        return test
    end testBridge
    
end script

